`timescale 1 ns/1 ps

module testbench;

localparam T = 10;
localparam N = 8;
localparam W = 4;

reg          test_clk;
reg  [W-1:0] test_addr_a, test_addr_b;
reg          test_we;
reg  [N-1:0] test_din_a;
wire [N-1:0] test_dout_a, test_dout_b;

integer i, j;

dp_ram_async_read
#(
   .N   (N        ),
   .W   (W        )
) uut (
   .clk   (test_clk   ),
   .we    (test_we    ),
   .addr_a(test_addr_a),
   .addr_b(test_addr_b),
   .din_a (test_din_a ),
   .dout_a(test_dout_a),
   .dout_b(test_dout_b)
);

always #T test_clk = ~test_clk;

initial begin
   test_clk = 0;
   test_we = 0;
   test_addr_a = 0;
   test_din_a = 0;
   j = 10;
   for (i=0; i<2**W; i=i+1) begin
      test_addr_a = i;
      test_din_a = j;
      j = j+5;
      @(negedge test_clk);
   end
   // write to register file
   test_we = 1;
   for (i=0; i<2**W; i=i+1) begin
      test_addr_a = i;
      test_din_a = j;
      j = j+5;
      @(negedge test_clk);
   end
   // read from register file
   test_we = 0;
   for (i=0; i<2**W; i=i+1) begin
      test_addr_a = i;
      test_addr_b = 2**W-1-i;
      @(negedge test_clk);
   end
   $stop;
end

endmodule