module fifo
#(
    parameter N = 8, // number of bits
              W = 4  // number of addr bits
) (
    input          clk, rstb,
    input          rd, wr,
    input  [N-1:0] w_data,
    output         empty, full,
    output [N-1:0] r_data
);

// signal declaration
reg  [N-1:0] register_file[2**W-1:0];
reg  [W-1:0] w_ptr_reg, w_ptr_next, w_ptr_succ;
reg  [W-1:0] r_ptr_reg, r_ptr_next, r_ptr_succ;
reg          full_reg, empty_reg, full_next, empty_next;
wire         wr_en;

// register file operation
always @(posedge clk) begin
    if (wr_en)
        register_file[w_ptr_reg] <= w_data;
end
assign r_data = register_file[r_ptr_reg];
assign wr_en = wr & ~full_reg;

// registers
always @(posedge clk or negedge rstb) begin
    if (~rstb) begin
        w_ptr_reg <= 0;
        r_ptr_reg <= 0;
        full_reg  <= 0;
        empty_reg <= 1;
    end
    else begin
        w_ptr_reg <= w_ptr_next;
        r_ptr_reg <= r_ptr_next;
        full_reg  <= full_next;
        empty_reg <= empty_next;
    end
end

// next-state logic
always @* begin
    w_ptr_succ = w_ptr_reg+1;
    r_ptr_succ = r_ptr_reg+1;
    // default values
    w_ptr_next = w_ptr_reg;
    r_ptr_next = r_ptr_reg;
    full_next = full_reg;
    empty_next = empty_reg;
    case ({wr, rd})
        // 2'b00: no operation
        2'b01: // read
            if (~empty_reg) begin
                r_ptr_next = r_ptr_succ;
                full_next = 0;
                if (r_ptr_next==w_ptr_reg) empty_next = 1;
            end
        2'b10: // write
            if (~full_reg) begin
                w_ptr_next = w_ptr_succ;
                empty_next = 0;
                if (w_ptr_next==r_ptr_reg) full_next = 1;
            end
        2'b11: // write and read
            begin
                w_ptr_next = w_ptr_succ;
                r_ptr_next = r_ptr_succ;
            end
    endcase
end

assign full = full_reg;
assign empty = empty_reg;

endmodule